/*
 * pixmaps.h - Include xpm pixmap files for pieces
 * $Id: pixmaps.h,v 1.1 2000/09/17 23:50:36 mann Exp $
 *
 * Copyright 1991 by Digital Equipment Corporation, Maynard, Massachusetts.
 * Enhancements Copyright 1992-2000 Free Software Foundation, Inc.
 *
 * The following terms apply to Digital Equipment Corporation's copyright
 * interest in XBoard:
 * ------------------------------------------------------------------------
 * All Rights Reserved
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the name of Digital not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.
 *
 * DIGITAL DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
 * ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
 * DIGITAL BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
 * ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
 * ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 * ------------------------------------------------------------------------
 *
 * The following terms apply to the enhanced version of XBoard distributed
 * by the Free Software Foundation:
 * ------------------------------------------------------------------------
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * ------------------------------------------------------------------------
 *
 * See the file ChangeLog for a revision history.
 */

#include "pll129.xpm"
#include "pld129.xpm"
#include "pdl129.xpm"
#include "pdd129.xpm"
#include "nll129.xpm"
#include "nld129.xpm"
#include "ndl129.xpm"
#include "ndd129.xpm"
#include "bll129.xpm"
#include "bld129.xpm"
#include "bdl129.xpm"
#include "bdd129.xpm"
#include "rll129.xpm"
#include "rld129.xpm"
#include "rdl129.xpm"
#include "rdd129.xpm"
#include "qll129.xpm"
#include "qld129.xpm"
#include "qdl129.xpm"
#include "qdd129.xpm"
#include "kll129.xpm"
#include "kld129.xpm"
#include "kdl129.xpm"
#include "kdd129.xpm"

#include "pll116.xpm"
#include "pld116.xpm"
#include "pdl116.xpm"
#include "pdd116.xpm"
#include "nll116.xpm"
#include "nld116.xpm"
#include "ndl116.xpm"
#include "ndd116.xpm"
#include "bll116.xpm"
#include "bld116.xpm"
#include "bdl116.xpm"
#include "bdd116.xpm"
#include "rll116.xpm"
#include "rld116.xpm"
#include "rdl116.xpm"
#include "rdd116.xpm"
#include "qll116.xpm"
#include "qld116.xpm"
#include "qdl116.xpm"
#include "qdd116.xpm"
#include "kll116.xpm"
#include "kld116.xpm"
#include "kdl116.xpm"
#include "kdd116.xpm"

#include "pll108.xpm"
#include "pld108.xpm"
#include "pdl108.xpm"
#include "pdd108.xpm"
#include "nll108.xpm"
#include "nld108.xpm"
#include "ndl108.xpm"
#include "ndd108.xpm"
#include "bll108.xpm"
#include "bld108.xpm"
#include "bdl108.xpm"
#include "bdd108.xpm"
#include "rll108.xpm"
#include "rld108.xpm"
#include "rdl108.xpm"
#include "rdd108.xpm"
#include "qll108.xpm"
#include "qld108.xpm"
#include "qdl108.xpm"
#include "qdd108.xpm"
#include "kll108.xpm"
#include "kld108.xpm"
#include "kdl108.xpm"
#include "kdd108.xpm"

#include "pll95.xpm"
#include "pld95.xpm"
#include "pdl95.xpm"
#include "pdd95.xpm"
#include "nll95.xpm"
#include "nld95.xpm"
#include "ndl95.xpm"
#include "ndd95.xpm"
#include "bll95.xpm"
#include "bld95.xpm"
#include "bdl95.xpm"
#include "bdd95.xpm"
#include "rll95.xpm"
#include "rld95.xpm"
#include "rdl95.xpm"
#include "rdd95.xpm"
#include "qll95.xpm"
#include "qld95.xpm"
#include "qdl95.xpm"
#include "qdd95.xpm"
#include "kll95.xpm"
#include "kld95.xpm"
#include "kdl95.xpm"
#include "kdd95.xpm"

#include "pll87.xpm"
#include "pld87.xpm"
#include "pdl87.xpm"
#include "pdd87.xpm"
#include "nll87.xpm"
#include "nld87.xpm"
#include "ndl87.xpm"
#include "ndd87.xpm"
#include "bll87.xpm"
#include "bld87.xpm"
#include "bdl87.xpm"
#include "bdd87.xpm"
#include "rll87.xpm"
#include "rld87.xpm"
#include "rdl87.xpm"
#include "rdd87.xpm"
#include "qll87.xpm"
#include "qld87.xpm"
#include "qdl87.xpm"
#include "qdd87.xpm"
#include "kll87.xpm"
#include "kld87.xpm"
#include "kdl87.xpm"
#include "kdd87.xpm"

#include "pll80.xpm"
#include "pld80.xpm"
#include "pdl80.xpm"
#include "pdd80.xpm"
#include "nll80.xpm"
#include "nld80.xpm"
#include "ndl80.xpm"
#include "ndd80.xpm"
#include "bll80.xpm"
#include "bld80.xpm"
#include "bdl80.xpm"
#include "bdd80.xpm"
#include "rll80.xpm"
#include "rld80.xpm"
#include "rdl80.xpm"
#include "rdd80.xpm"
#include "qll80.xpm"
#include "qld80.xpm"
#include "qdl80.xpm"
#include "qdd80.xpm"
#include "kll80.xpm"
#include "kld80.xpm"
#include "kdl80.xpm"
#include "kdd80.xpm"

#include "pll72.xpm"
#include "pld72.xpm"
#include "pdl72.xpm"
#include "pdd72.xpm"
#include "nll72.xpm"
#include "nld72.xpm"
#include "ndl72.xpm"
#include "ndd72.xpm"
#include "bll72.xpm"
#include "bld72.xpm"
#include "bdl72.xpm"
#include "bdd72.xpm"
#include "rll72.xpm"
#include "rld72.xpm"
#include "rdl72.xpm"
#include "rdd72.xpm"
#include "qll72.xpm"
#include "qld72.xpm"
#include "qdl72.xpm"
#include "qdd72.xpm"
#include "kll72.xpm"
#include "kld72.xpm"
#include "kdl72.xpm"
#include "kdd72.xpm"

#include "pll64.xpm"
#include "pld64.xpm"
#include "pdl64.xpm"
#include "pdd64.xpm"
#include "nll64.xpm"
#include "nld64.xpm"
#include "ndl64.xpm"
#include "ndd64.xpm"
#include "bll64.xpm"
#include "bld64.xpm"
#include "bdl64.xpm"
#include "bdd64.xpm"
#include "rll64.xpm"
#include "rld64.xpm"
#include "rdl64.xpm"
#include "rdd64.xpm"
#include "qll64.xpm"
#include "qld64.xpm"
#include "qdl64.xpm"
#include "qdd64.xpm"
#include "kll64.xpm"
#include "kld64.xpm"
#include "kdl64.xpm"
#include "kdd64.xpm"

#include "pll58.xpm"
#include "pld58.xpm"
#include "pdl58.xpm"
#include "pdd58.xpm"
#include "nll58.xpm"
#include "nld58.xpm"
#include "ndl58.xpm"
#include "ndd58.xpm"
#include "bll58.xpm"
#include "bld58.xpm"
#include "bdl58.xpm"
#include "bdd58.xpm"
#include "rll58.xpm"
#include "rld58.xpm"
#include "rdl58.xpm"
#include "rdd58.xpm"
#include "qll58.xpm"
#include "qld58.xpm"
#include "qdl58.xpm"
#include "qdd58.xpm"
#include "kll58.xpm"
#include "kld58.xpm"
#include "kdl58.xpm"
#include "kdd58.xpm"

#include "pll54.xpm"
#include "pld54.xpm"
#include "pdl54.xpm"
#include "pdd54.xpm"
#include "nll54.xpm"
#include "nld54.xpm"
#include "ndl54.xpm"
#include "ndd54.xpm"
#include "bll54.xpm"
#include "bld54.xpm"
#include "bdl54.xpm"
#include "bdd54.xpm"
#include "rll54.xpm"
#include "rld54.xpm"
#include "rdl54.xpm"
#include "rdd54.xpm"
#include "qll54.xpm"
#include "qld54.xpm"
#include "qdl54.xpm"
#include "qdd54.xpm"
#include "kll54.xpm"
#include "kld54.xpm"
#include "kdl54.xpm"
#include "kdd54.xpm"

#include "pll49.xpm"
#include "pld49.xpm"
#include "pdl49.xpm"
#include "pdd49.xpm"
#include "nll49.xpm"
#include "nld49.xpm"
#include "ndl49.xpm"
#include "ndd49.xpm"
#include "bll49.xpm"
#include "bld49.xpm"
#include "bdl49.xpm"
#include "bdd49.xpm"
#include "rll49.xpm"
#include "rld49.xpm"
#include "rdl49.xpm"
#include "rdd49.xpm"
#include "qll49.xpm"
#include "qld49.xpm"
#include "qdl49.xpm"
#include "qdd49.xpm"
#include "kll49.xpm"
#include "kld49.xpm"
#include "kdl49.xpm"
#include "kdd49.xpm"

#include "pll45.xpm"
#include "pld45.xpm"
#include "pdl45.xpm"
#include "pdd45.xpm"
#include "nll45.xpm"
#include "nld45.xpm"
#include "ndl45.xpm"
#include "ndd45.xpm"
#include "bll45.xpm"
#include "bld45.xpm"
#include "bdl45.xpm"
#include "bdd45.xpm"
#include "rll45.xpm"
#include "rld45.xpm"
#include "rdl45.xpm"
#include "rdd45.xpm"
#include "qll45.xpm"
#include "qld45.xpm"
#include "qdl45.xpm"
#include "qdd45.xpm"
#include "kll45.xpm"
#include "kld45.xpm"
#include "kdl45.xpm"
#include "kdd45.xpm"

#include "pll40.xpm"
#include "pld40.xpm"
#include "pdl40.xpm"
#include "pdd40.xpm"
#include "nll40.xpm"
#include "nld40.xpm"
#include "ndl40.xpm"
#include "ndd40.xpm"
#include "bll40.xpm"
#include "bld40.xpm"
#include "bdl40.xpm"
#include "bdd40.xpm"
#include "rll40.xpm"
#include "rld40.xpm"
#include "rdl40.xpm"
#include "rdd40.xpm"
#include "qll40.xpm"
#include "qld40.xpm"
#include "qdl40.xpm"
#include "qdd40.xpm"
#include "kll40.xpm"
#include "kld40.xpm"
#include "kdl40.xpm"
#include "kdd40.xpm"

#include "pll37.xpm"
#include "pld37.xpm"
#include "pdl37.xpm"
#include "pdd37.xpm"
#include "nll37.xpm"
#include "nld37.xpm"
#include "ndl37.xpm"
#include "ndd37.xpm"
#include "bll37.xpm"
#include "bld37.xpm"
#include "bdl37.xpm"
#include "bdd37.xpm"
#include "rll37.xpm"
#include "rld37.xpm"
#include "rdl37.xpm"
#include "rdd37.xpm"
#include "qll37.xpm"
#include "qld37.xpm"
#include "qdl37.xpm"
#include "qdd37.xpm"
#include "kll37.xpm"
#include "kld37.xpm"
#include "kdl37.xpm"
#include "kdd37.xpm"

#include "pll33.xpm"
#include "pld33.xpm"
#include "pdl33.xpm"
#include "pdd33.xpm"
#include "nll33.xpm"
#include "nld33.xpm"
#include "ndl33.xpm"
#include "ndd33.xpm"
#include "bll33.xpm"
#include "bld33.xpm"
#include "bdl33.xpm"
#include "bdd33.xpm"
#include "rll33.xpm"
#include "rld33.xpm"
#include "rdl33.xpm"
#include "rdd33.xpm"
#include "qll33.xpm"
#include "qld33.xpm"
#include "qdl33.xpm"
#include "qdd33.xpm"
#include "kll33.xpm"
#include "kld33.xpm"
#include "kdl33.xpm"
#include "kdd33.xpm"

#include "pll29.xpm"
#include "pld29.xpm"
#include "pdl29.xpm"
#include "pdd29.xpm"
#include "nll29.xpm"
#include "nld29.xpm"
#include "ndl29.xpm"
#include "ndd29.xpm"
#include "bll29.xpm"
#include "bld29.xpm"
#include "bdl29.xpm"
#include "bdd29.xpm"
#include "rll29.xpm"
#include "rld29.xpm"
#include "rdl29.xpm"
#include "rdd29.xpm"
#include "qll29.xpm"
#include "qld29.xpm"
#include "qdl29.xpm"
#include "qdd29.xpm"
#include "kll29.xpm"
#include "kld29.xpm"
#include "kdl29.xpm"
#include "kdd29.xpm"

#include "pll25.xpm"
#include "pld25.xpm"
#include "pdl25.xpm"
#include "pdd25.xpm"
#include "nll25.xpm"
#include "nld25.xpm"
#include "ndl25.xpm"
#include "ndd25.xpm"
#include "bll25.xpm"
#include "bld25.xpm"
#include "bdl25.xpm"
#include "bdd25.xpm"
#include "rll25.xpm"
#include "rld25.xpm"
#include "rdl25.xpm"
#include "rdd25.xpm"
#include "qll25.xpm"
#include "qld25.xpm"
#include "qdl25.xpm"
#include "qdd25.xpm"
#include "kll25.xpm"
#include "kld25.xpm"
#include "kdl25.xpm"
#include "kdd25.xpm"

#include "pll21.xpm"
#include "pld21.xpm"
#include "pdl21.xpm"
#include "pdd21.xpm"
#include "nll21.xpm"
#include "nld21.xpm"
#include "ndl21.xpm"
#include "ndd21.xpm"
#include "bll21.xpm"
#include "bld21.xpm"
#include "bdl21.xpm"
#include "bdd21.xpm"
#include "rll21.xpm"
#include "rld21.xpm"
#include "rdl21.xpm"
#include "rdd21.xpm"
#include "qll21.xpm"
#include "qld21.xpm"
#include "qdl21.xpm"
#include "qdd21.xpm"
#include "kll21.xpm"
#include "kld21.xpm"
#include "kdl21.xpm"
#include "kdd21.xpm"

typedef struct {
  int size;
  char **xpm[6][4];
} XpmPieces;

XpmPieces builtInXpms[] = {\
  { 129, {\
    { pll129, pld129, pdl129, pdd129, },\
    { nll129, nld129, ndl129, ndd129, },\
    { bll129, bld129, bdl129, bdd129, },\
    { rll129, rld129, rdl129, rdd129, },\
    { qll129, qld129, qdl129, qdd129, },\
    { kll129, kld129, kdl129, kdd129, },\
  } },\
  { 116, {\
    { pll116, pld116, pdl116, pdd116, },\
    { nll116, nld116, ndl116, ndd116, },\
    { bll116, bld116, bdl116, bdd116, },\
    { rll116, rld116, rdl116, rdd116, },\
    { qll116, qld116, qdl116, qdd116, },\
    { kll116, kld116, kdl116, kdd116, },\
  } },\
  { 108, {\
    { pll108, pld108, pdl108, pdd108, },\
    { nll108, nld108, ndl108, ndd108, },\
    { bll108, bld108, bdl108, bdd108, },\
    { rll108, rld108, rdl108, rdd108, },\
    { qll108, qld108, qdl108, qdd108, },\
    { kll108, kld108, kdl108, kdd108, },\
  } },\
  { 95, {\
    { pll95, pld95, pdl95, pdd95, },\
    { nll95, nld95, ndl95, ndd95, },\
    { bll95, bld95, bdl95, bdd95, },\
    { rll95, rld95, rdl95, rdd95, },\
    { qll95, qld95, qdl95, qdd95, },\
    { kll95, kld95, kdl95, kdd95, },\
  } },\
  { 87, {\
    { pll87, pld87, pdl87, pdd87, },\
    { nll87, nld87, ndl87, ndd87, },\
    { bll87, bld87, bdl87, bdd87, },\
    { rll87, rld87, rdl87, rdd87, },\
    { qll87, qld87, qdl87, qdd87, },\
    { kll87, kld87, kdl87, kdd87, },\
  } },\
  { 80, {\
    { pll80, pld80, pdl80, pdd80, },\
    { nll80, nld80, ndl80, ndd80, },\
    { bll80, bld80, bdl80, bdd80, },\
    { rll80, rld80, rdl80, rdd80, },\
    { qll80, qld80, qdl80, qdd80, },\
    { kll80, kld80, kdl80, kdd80, },\
  } },\
  { 72, {\
    { pll72, pld72, pdl72, pdd72, },\
    { nll72, nld72, ndl72, ndd72, },\
    { bll72, bld72, bdl72, bdd72, },\
    { rll72, rld72, rdl72, rdd72, },\
    { qll72, qld72, qdl72, qdd72, },\
    { kll72, kld72, kdl72, kdd72, },\
  } },\
  { 64, {\
    { pll64, pld64, pdl64, pdd64, },\
    { nll64, nld64, ndl64, ndd64, },\
    { bll64, bld64, bdl64, bdd64, },\
    { rll64, rld64, rdl64, rdd64, },\
    { qll64, qld64, qdl64, qdd64, },\
    { kll64, kld64, kdl64, kdd64, },\
  } },\
  { 58, {\
    { pll58, pld58, pdl58, pdd58, },\
    { nll58, nld58, ndl58, ndd58, },\
    { bll58, bld58, bdl58, bdd58, },\
    { rll58, rld58, rdl58, rdd58, },\
    { qll58, qld58, qdl58, qdd58, },\
    { kll58, kld58, kdl58, kdd58, },\
  } },\
  { 54, {\
    { pll54, pld54, pdl54, pdd54, },\
    { nll54, nld54, ndl54, ndd54, },\
    { bll54, bld54, bdl54, bdd54, },\
    { rll54, rld54, rdl54, rdd54, },\
    { qll54, qld54, qdl54, qdd54, },\
    { kll54, kld54, kdl54, kdd54, },\
  } },\
  { 49, {\
    { pll49, pld49, pdl49, pdd49, },\
    { nll49, nld49, ndl49, ndd49, },\
    { bll49, bld49, bdl49, bdd49, },\
    { rll49, rld49, rdl49, rdd49, },\
    { qll49, qld49, qdl49, qdd49, },\
    { kll49, kld49, kdl49, kdd49, },\
  } },\
  { 45, {\
    { pll45, pld45, pdl45, pdd45, },\
    { nll45, nld45, ndl45, ndd45, },\
    { bll45, bld45, bdl45, bdd45, },\
    { rll45, rld45, rdl45, rdd45, },\
    { qll45, qld45, qdl45, qdd45, },\
    { kll45, kld45, kdl45, kdd45, },\
  } },\
  { 40, {\
    { pll40, pld40, pdl40, pdd40, },\
    { nll40, nld40, ndl40, ndd40, },\
    { bll40, bld40, bdl40, bdd40, },\
    { rll40, rld40, rdl40, rdd40, },\
    { qll40, qld40, qdl40, qdd40, },\
    { kll40, kld40, kdl40, kdd40, },\
  } },\
  { 37, {\
    { pll37, pld37, pdl37, pdd37, },\
    { nll37, nld37, ndl37, ndd37, },\
    { bll37, bld37, bdl37, bdd37, },\
    { rll37, rld37, rdl37, rdd37, },\
    { qll37, qld37, qdl37, qdd37, },\
    { kll37, kld37, kdl37, kdd37, },\
  } },\
  { 33, {\
    { pll33, pld33, pdl33, pdd33, },\
    { nll33, nld33, ndl33, ndd33, },\
    { bll33, bld33, bdl33, bdd33, },\
    { rll33, rld33, rdl33, rdd33, },\
    { qll33, qld33, qdl33, qdd33, },\
    { kll33, kld33, kdl33, kdd33, },\
  } },\
  { 29, {\
    { pll29, pld29, pdl29, pdd29, },\
    { nll29, nld29, ndl29, ndd29, },\
    { bll29, bld29, bdl29, bdd29, },\
    { rll29, rld29, rdl29, rdd29, },\
    { qll29, qld29, qdl29, qdd29, },\
    { kll29, kld29, kdl29, kdd29, },\
  } },\
  { 25, {\
    { pll25, pld25, pdl25, pdd25, },\
    { nll25, nld25, ndl25, ndd25, },\
    { bll25, bld25, bdl25, bdd25, },\
    { rll25, rld25, rdl25, rdd25, },\
    { qll25, qld25, qdl25, qdd25, },\
    { kll25, kld25, kdl25, kdd25, },\
  } },\
  { 21, {\
    { pll21, pld21, pdl21, pdd21, },\
    { nll21, nld21, ndl21, ndd21, },\
    { bll21, bld21, bdl21, bdd21, },\
    { rll21, rld21, rdl21, rdd21, },\
    { qll21, qld21, qdl21, qdd21, },\
    { kll21, kld21, kdl21, kdd21, },\
  } },\
  { 0, {\
    { NULL, NULL, NULL, NULL, },\
    { NULL, NULL, NULL, NULL, },\
    { NULL, NULL, NULL, NULL, },\
    { NULL, NULL, NULL, NULL, },\
    { NULL, NULL, NULL, NULL, },\
    { NULL, NULL, NULL, NULL, },\
  } },\
};
